package main.java;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

public class ChiSquareMutualInformation {
    private Map<Bigram, Double> termMatrix = new HashMap<Bigram, Double>();
    private int n;
    private double docsContainingXY;
    private double docsContainingX;
    private double docsContainingY;
    private List<String> allTerms = new ArrayList<String>();

    public ChiSquareMutualInformation(List<String> abstracts){
        n = abstracts.size();
        for(int i = 0; i < abstracts.size(); i++){
            String[] line = abstracts.get(i).split(" ");
            for(String word: line){
                if(word.contains("/") || word.contains("\\") || word.contains("{") || word.contains("}")) {
                    continue;
                }
                else if(word.length() < 5){
                    continue;
                }
                else{
                    if(!Utils.containsDigits(word)){
                        allTerms.add(word);
                    }
                }
            }
        }
        Collections.sort(allTerms);
        sortAndCleanTerms();
        getMutualInformation(abstracts);
    }

    private void sortAndCleanTerms() {
        int counter = 1;
        List<String> tempTerms = new ArrayList<String>();
        String termX = "";
        tempTerms.addAll(allTerms);
        for(int i = 0; i < allTerms.size()-1; i++){
            if(allTerms.get(i).equals(termX)){
                continue;
            }
            else{
                termX = allTerms.get(i);
                for(int j = i + 1; j < allTerms.size(); j++){
                    if(termX.equals(allTerms.get(j))){
                        counter++;
                    }
                }
                if(n == 10){
                    if(counter < 1){
                        while(tempTerms.remove(termX));
                    }
                }
                else if(n == 75){
                    if(counter < 3){
                        while(tempTerms.remove(termX));
                    }
                }
                else{
                    if(counter < 5){
                        while(tempTerms.remove(termX));
                    }
                }
            }
            counter = 1;
        }
        Set<String> uniqueTerms = new HashSet<String>();
        uniqueTerms.addAll(tempTerms);
        allTerms.clear();
        allTerms.addAll(uniqueTerms);
    }

    private void getMutualInformation(List<String> abstracts){
        double n = this.n;
        double sum;

        for(int f1 = 0; f1 < allTerms.size()-1; f1++){
            Bigram bigram = null;
            for(int f2 = f1 + 1; f2 < allTerms.size(); f2++){
                bigram = new Bigram(allTerms.get(f1), allTerms.get(f2));
                buildOccurrencesInDocs(bigram, abstracts);
                double p_xy = docsContainingXY / n;
                double p_x = docsContainingX / n;
                double p_y = docsContainingY / n;

                if(p_xy == 0.0 || p_x == 0.0 || p_y == 0.0){
                    termMatrix.put(bigram, -30000.0);
                }
                else{
                    double power = (p_xy - ((double)1)) / (n * p_x * p_y);
                    power = power * power;
                    sum = power / (p_x * p_y);
                    sum = sum * (-1);
                    /*sum = (-1) * (
                            (p_xy) * Utils.logWithoutNaN(
                                    p_xy / (p_x * p_y)
                            )
                    );*/
                    termMatrix.put(bigram, sum);
                }
            }
        }
    }

    private void buildOccurrencesInDocs(Bigram bigram, List<String> abstracts){
        String x = bigram.getFirst();
        String y = bigram.getLast();
        docsContainingX = 0;
        docsContainingXY = 0;
        docsContainingY = 0;
        for(String line: abstracts){
            if(line.contains(x) && line.contains(y)){
                docsContainingXY++;
            }
            if(line.contains(x)){
                docsContainingX++;
            }
            if(line.contains(y)){
                docsContainingY++;
            }
        }
    }

    public void printMatrix(){
        int lines = 0;
        for(Map.Entry<Bigram, Double> item: termMatrix.entrySet()){
            System.out.println(item.getKey() + ": " + item.getValue());
            lines++;
            if(lines >= 1000){
                break;
            }
        }
    }

    public void sortMatrix(){
        DoubleValueComparator valueComparator = new DoubleValueComparator(termMatrix);
        TreeMap<Bigram, Double> sortedMap = new TreeMap<Bigram, Double>(valueComparator);
        sortedMap.putAll(termMatrix);
        termMatrix = sortedMap;
    }

    public List<String> getNTopTerms(int n, String query){
        List<String> topTerms = new ArrayList<String>();
        topTerms.add(query);
        for(Bigram bigram: termMatrix.keySet()){
            if(!topTerms.contains(bigram.getFirst())){
                topTerms.add(bigram.getFirst());
            }
            if(!topTerms.contains(bigram.getLast())){
                topTerms.add(bigram.getLast());
            }
            if(topTerms.size() >= n+1){
                break;
            }
        }
        return topTerms;
    }
}
