package main.java;

import org.apache.lucene.analysis.core.WhitespaceAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.TextField;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.search.similarities.LMJelinekMercerSimilarity;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.NIOFSDirectory;
import org.apache.lucene.util.Version;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

/**
 * Created with IntelliJ IDEA.
 * User: Geir
 * Date: 8/12/13
 * Time: 14:10
 * To change this template use File | Settings | File Templates.
 */
public class IndexConstructor {
    public static final String ABSTRACTS_FILE = "/Users/Geir/Development/master/src/main/resources/long_abstracts_en.nt";
    public static final String IMAGES_PATH = "/Users/Geir/Development/master/src/main/resources/images/";
    public static final String META_TAGS_PATH = "/Users/Geir/Development/master/src/main/resources/images/meta/tags/";

    //INDEXES
    //*TAGS
    //**TFIDF
    public static final String TFIDF_TAGS_INDEX_UNPROCESSED = "/Users/Geir/Development/master/src/main/resources/ImageIndex/TF-IDF/Unprocessed/";
    public static final String TFIDF_TAGS_INDEX_STOPWORD = "/Users/Geir/Development/master/src/main/resources/ImageIndex/TF-IDF/JustStopWordRemoval/";
    public static final String TFIDF_TAGS_INDEX_STEMMED = "/Users/Geir/Development/master/src/main/resources/ImageIndex/TF-IDF/JustStemming/";
    public static final String TFIDF_TAGS_INDEX_STEMMED_STOPWORD = "/Users/Geir/Development/master/src/main/resources/ImageIndex/TF-IDF/StemmingAndStopWordRemoval/";

    //**BM25
    public static final String BM25_TAGS_INDEX_UNPROCESSED = "/Users/Geir/Development/master/src/main/resources/ImageIndex/BM25/Unprocessed/";
    public static final String BM25_TAGS_INDEX_STOPWORD = "/Users/Geir/Development/master/src/main/resources/ImageIndex/BM25/JustStopWordRemoval/";
    public static final String BM25_TAGS_INDEX_STEMMED = "/Users/Geir/Development/master/src/main/resources/ImageIndex/BM25/JustStemming/";
    public static final String BM25_TAGS_INDEX_STEMMED_STOPWORD = "/Users/Geir/Development/master/src/main/resources/ImageIndex/BM25/StemmingAndStopWordRemoval";

    //**LANGUAGE MODEL
    public static final String LM_TAGS_INDEX_UNPROCESSED = "/Users/Geir/Development/master/src/main/resources/ImageIndex/LM/Unprocessed/";
    public static final String LM_TAGS_INDEX_STOPWORD = "/Users/Geir/Development/master/src/main/resources/ImageIndex/LM/JustStopWordRemoval/";
    public static final String LM_TAGS_INDEX_STEMMED = "/Users/Geir/Development/master/src/main/resources/ImageIndex/LM/JustStemming/";
    public static final String LM_TAGS_INDEX_STEMMED_STOPWORD = "/Users/Geir/Development/master/src/main/resources/ImageIndex/LM/StemmingAndStopWordRemoval";

    //*ABSTRACTS
    //**TF-IDF
    public static final String TFIDF_ABSTRACTS_INDEX_UNPROCESSED = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/TF-IDF/Unprocessed/";
    public static final String TFIDF_ABSTRACTS_INDEX_STOPWORD = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/TF-IDF/JustStopWordRemoval/";
    public static final String TFIDF_ABSTRACTS_INDEX_STEMMED = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/TF-IDF/JustStemming/";
    public static final String TFIDF_ABSTRACTS_INDEX_STEMMED_STOPWORD = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/TF-IDF/StemmingAndStopWordRemoval/";

    //**BM25
    public static final String BM25_ABSTRACTS_INDEX_UNPROCESSED = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/BM25/Unprocessed/";
    public static final String BM25_ABSTRACTS_INDEX_STOPWORD = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/BM25/JustStopWordRemoval/";
    public static final String BM25_ABSTRACTS_INDEX_STEMMED = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/BM25/JustStemming/";
    public static final String BM25_ABSTRACTS_INDEX_STEMMED_STOPWORD = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/BM25/StemmingAndStopWordRemoval/";

    //**LANGUAGE MODEL
    public static final String LM_ABSTRACTS_INDEX_UNPROCESSED = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/LM/Unprocessed";
    public static final String LM_ABSTRACTS_INDEX_STOPWORD = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/LM/JustStopWordRemoval/";
    public static final String LM_ABSTRACTS_INDEX_STEMMED = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/LM/JustStemming/";
    public static final String LM_ABSTRACTS_INDEX_STEMMED_STOPWORD = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/LM/StemmingAndStopWordRemoval/";

    public static void ProcessTags() throws IOException {
        //NIOFSDirectory not recommended for Windows because of a bug in how FileChannel.read is implemented in Sun's JRE.
        //For Windows, use FSDirectory instead.
        Directory index = new NIOFSDirectory(new File(LM_TAGS_INDEX_STEMMED_STOPWORD));

        List<String> stopwords = Utils.createStopWordList();

        WhitespaceAnalyzer analyzer = new WhitespaceAnalyzer(Version.LUCENE_41);
        IndexWriterConfig config = new IndexWriterConfig(Version.LUCENE_41, analyzer);

        //config.setSimilarity(new BM25Similarity());
        config.setSimilarity(new LMJelinekMercerSimilarity((float)0.1));
        config.setOpenMode(IndexWriterConfig.OpenMode.CREATE_OR_APPEND);

        FileInputStream fis = null;
        DataInputStream dis = null;
        BufferedReader br = null;
        String strLine;
        String tags = "";
        double fileNumber = 1;
        File directory = new File(META_TAGS_PATH);
        IndexWriter writer = new IndexWriter(index, config);

        String percentComplete = "0.0";

        for (File subDirectory : directory.listFiles()){
            for(File file : subDirectory.listFiles()){
                try{
                    fis = new FileInputStream(file.getAbsolutePath());
                    dis = new DataInputStream(fis);
                    br = new BufferedReader(new InputStreamReader(dis));
                } catch (IOException e){
                    System.err.println("Error: " + e.getMessage());
                }

                while((strLine = br.readLine()) != null){
                    if(!stopwords.contains(strLine)){
                        tags += Utils.stemWord(strLine) + " ";
                        //tags += strLine + " ";
                    }
                }

                if(tags.length() > 1){
                    tags = tags.substring(0,tags.length()-1);
                }

                String imageName = file.getName();
                imageName = imageName.substring(0,imageName.indexOf("."));
                imageName += ".jpg";

                Document doc = new Document();
                doc.add(new TextField("image", imageName, Field.Store.YES));
                doc.add(new TextField("tags", tags, Field.Store.YES));

                percentComplete = Utils.percentageCompleteCalc(fileNumber, percentComplete, subDirectory);

                fileNumber++;

                writer.addDocument(doc);

                tags = "";
                br.close();
                dis.close();
                fis.close();
            }
        }
        writer.close();
    }

    public static void ProcessAbstracts() throws IOException{
        FileInputStream fis;
        DataInputStream dis;
        BufferedReader br = null;
        Scanner scan;
        String strLine;				        //To be: One line from article file. One line = all details from one article
        String article = new String("");	//To be: Word by word from strLine
        int maxNumberOfLines = 1000; 		//Number of articles to be traversed before halt. Primarily for testing/developing
        int numberOfLinesSoFar = 0;			//Number of articles processed thus far
        int deprecatedWords = 0;
        String percentComplete = "";		//Previous percentage update to avoid being spammed by "x percent complete"

        ArrayList<String> queue = new ArrayList<String>(); //ArrayList where we store every non-stopword uniquely from every article
        DecimalFormat format = new DecimalFormat("#.00");  //Formatting for percentage complete display

        //NIOFSDirectory not recommended for Windows because of a bug in how FileChannel.read is implemented in Sun's JRE.
        //For Windows, use FSDirectory instead.
        Directory index = new NIOFSDirectory(new File(LM_ABSTRACTS_INDEX_STEMMED_STOPWORD));
        WhitespaceAnalyzer analyzer = new WhitespaceAnalyzer(Version.LUCENE_41);
        IndexWriterConfig config = new IndexWriterConfig(Version.LUCENE_41, analyzer);

        //Set similiarity model. Comment out for default TF-IDF
        config.setSimilarity(new LMJelinekMercerSimilarity((float)0.1));
        //config.setSimilarity(new BM25Similarity());
        config.setOpenMode(IndexWriterConfig.OpenMode.CREATE_OR_APPEND);

        IndexWriter writer = new IndexWriter(index, config);
        List<String> stopwords = Utils.createStopWordList();

        try{
            fis = new FileInputStream(ABSTRACTS_FILE);
            dis = new DataInputStream(fis);
            br = new BufferedReader(new InputStreamReader(dis));
        }catch (IOException e){
            System.err.println("Error: " + e.getMessage());
        }

        //&& (numberOfLinesSoFar < maxNumberOfLines)
        while(((strLine = br.readLine()) != null)){
            if(Utils.lineIsNotCommentOrLineChange(strLine)){
                scan = new Scanner(strLine);
                while(scan.hasNext()){
                    String word = scan.next();
                    if(!(word.matches("[0-9]+"))){
                        if(word.contains("http://dbpedia.org/resource")){
                            if(!queue.isEmpty()){
                                addArticle(article, queue, writer);
                                percentComplete = Utils.percentageCompleteCalc(numberOfLinesSoFar, percentComplete);
                            }
                            article = word.substring(29,word.length()-1);
                            article = article.replaceAll("_"," ");
                            word = article.toLowerCase();
                        }
                        else if(word.contains("http://dbpedia.org/ontology/abstract")){
//								do nothing
                        }
                        else{
                            word = word.toLowerCase();
                            word = Utils.cleanWord(word);
                            if(!word.equals("")){
                                if(!stopwords.contains(word)){
                                    word = Utils.stemWord(word);
                                    if(word.contains("\\u")){
                                        Utils.convertUnicode(word);
                                    }
                                    if(!word.equals("")){
                                        queue.add(word);
                                    }
                                }
                            }
                        }
                    }
                }
                numberOfLinesSoFar++;
            }//end if(!strLine.contains()
        }//while(strLine = br.readLine())
        writer.close();
        System.out.println("Completed!");
    }

    private static String addArticle(String article, ArrayList<String> queue, IndexWriter writer) throws IOException {
        Document doc = new Document();
        String line = "";
        doc.add(new TextField("article",article.toLowerCase(), Field.Store.YES));
        for(int i = 0; i < queue.size(); i++){
            line += queue.get(i) + " ";
        }

        doc.add(new TextField("desc", line, Field.Store.YES));
        writer.addDocument(doc);

        queue.clear();
        line = "";
        return line;
    }
}
