package main.java;

import org.apache.lucene.analysis.core.WhitespaceAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.queryparser.classic.ParseException;
import org.apache.lucene.queryparser.classic.QueryParser;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopScoreDocCollector;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.NIOFSDirectory;
import org.apache.lucene.util.Version;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Created with IntelliJ IDEA.
 * User: Geir
 * Date: 8/12/13
 * Time: 17:10
 * To change this template use File | Settings | File Templates.
 */
public class Searcher {
    //INDEXES
    //*TAGS
    //**TFIDF
    public static final String TFIDF_TAGS_INDEX_UNPROCESSED = "/Users/Geir/Development/master/src/main/resources/ImageIndex/TF-IDF/Unprocessed/";
    public static final String TFIDF_TAGS_INDEX_STOPWORD = "/Users/Geir/Development/master/src/main/resources/ImageIndex/TF-IDF/JustStopWordRemoval/";
    public static final String TFIDF_TAGS_INDEX_STEMMED = "/Users/Geir/Development/master/src/main/resources/ImageIndex/TF-IDF/JustStemming/";
    public static final String TFIDF_TAGS_INDEX_STEMMED_STOPWORD = "/Users/Geir/Development/master/src/main/resources/ImageIndex/TF-IDF/StemmingAndStopWordRemoval/";

    //**BM25
    public static final String BM25_TAGS_INDEX_UNPROCESSED = "/Users/Geir/Development/master/src/main/resources/ImageIndex/BM25/Unprocessed/";
    public static final String BM25_TAGS_INDEX_STOPWORD = "/Users/Geir/Development/master/src/main/resources/ImageIndex/BM25/JustStopWordRemoval/";
    public static final String BM25_TAGS_INDEX_STEMMED = "/Users/Geir/Development/master/src/main/resources/ImageIndex/BM25/JustStemming/";
    public static final String BM25_TAGS_INDEX_STEMMED_STOPWORD = "/Users/Geir/Development/master/src/main/resources/ImageIndex/BM25/StemmingAndStopWordRemoval";

    //**LANGUAGE MODEL
    public static final String LM_TAGS_INDEX_UNPROCESSED = "/Users/Geir/Development/master/src/main/resources/ImageIndex/LM/Unprocessed/";
    public static final String LM_TAGS_INDEX_STOPWORD = "/Users/Geir/Development/master/src/main/resources/ImageIndex/LM/JustStopWordRemoval/";
    public static final String LM_TAGS_INDEX_STEMMED = "/Users/Geir/Development/master/src/main/resources/ImageIndex/LM/JustStemming/";
    public static final String LM_TAGS_INDEX_STEMMED_STOPWORD = "/Users/Geir/Development/master/src/main/resources/ImageIndex/LM/StemmingAndStopWordRemoval";

    //*ABSTRACTS
    //**TF-IDF
    public static final String TFIDF_ABSTRACTS_INDEX_UNPROCESSED = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/TF-IDF/Unprocessed/";
    public static final String TFIDF_ABSTRACTS_INDEX_STOPWORD = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/TF-IDF/JustStopWordRemoval/";
    public static final String TFIDF_ABSTRACTS_INDEX_STEMMED = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/TF-IDF/JustStemming/";
    public static final String TFIDF_ABSTRACTS_INDEX_STEMMED_STOPWORD = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/TF-IDF/StemmingAndStopWordRemoval/";

    //**BM25
    public static final String BM25_ABSTRACTS_INDEX_UNPROCESSED = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/BM25/Unprocessed/";
    public static final String BM25_ABSTRACTS_INDEX_STOPWORD = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/BM25/JustStopWordRemoval/";
    public static final String BM25_ABSTRACTS_INDEX_STEMMED = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/BM25/JustStemming/";
    public static final String BM25_ABSTRACTS_INDEX_STEMMED_STOPWORD = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/BM25/StemmingAndStopWordRemoval/";

    //**LANGUAGE MODEL
    public static final String LM_ABSTRACTS_INDEX_UNPROCESSED = "/Users/Geir/Development/master/src/main/resources/LM/Unprocessed";
    public static final String LM_ABSTRACTS_INDEX_STOPWORD = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/LM/JustStopWordRemoval/";
    public static final String LM_ABSTRACTS_INDEX_STEMMED = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/LM/JustStemming/";
    public static final String LM_ABSTRACTS_INDEX_STEMMED_STOPWORD = "/Users/Geir/Development/master/src/main/resources/AbstractsIndex/LM/StemmingAndStopWordRemoval/";

    public static void searchTags(String input, int hitsPerPage, boolean print) throws ParseException, IOException{
        WhitespaceAnalyzer analyzer = new WhitespaceAnalyzer(Version.LUCENE_41);
        Query q = new QueryParser(Version.LUCENE_41, "tags", analyzer).parse(input);
        Directory index = new NIOFSDirectory(new File(TFIDF_TAGS_INDEX_STOPWORD));
        List<String> result = new ArrayList<String>();
        List<String> tags = new ArrayList<String>();
        List<String> queryTerms = new ArrayList<String>();
        queryTerms = Utils.stringToSortedList(input);

        String tagsString = "";
        IndexReader reader = IndexReader.open(index);

        IndexSearcher searcher = new IndexSearcher(reader);
        //searcher.setSimilarity(new BM25Similarity());
        //searcher.setSimilarity(new LMJelinekMercerSimilarity((float)0.1));

        TopScoreDocCollector collector = TopScoreDocCollector.create(hitsPerPage, true);
        searcher.search(q, collector);
        ScoreDoc[] hits = collector.topDocs().scoreDocs;

        String[] tagTerms = input.split(" ");

        //System.out.println("Found " + hits.length + " images.");
        System.out.println("\\begin{longtable}{| c | c | c |}");
        System.out.println("\t\\caption{Results of image retrieval using NT for query = \""+ tagTerms[0] +"\"}");
        System.out.println("\t\\label{tab:resultsIR:nt:"+ tagTerms[0] +"}\\\\");
        System.out.println("\t\\hline");
        System.out.println("\t\\multicolumn{3}{| c |}{Query: "+ tagTerms[0] +"}\\\\");
        System.out.println("\t\\multicolumn{3}{| c |}{ET: "+ tagTerms[1] + " " + tagTerms[2] + " " + tagTerms[3] + " " + tagTerms[4] + " " + tagTerms[5] +"}\\\\");
        System.out.println("\t\\hline");
        System.out.println("\tImage \\# & Related image? & Contains ET \\\\");
        System.out.println("\t\\hline");

        for(int i=0;i<hits.length;++i) {
            int docId = hits[i].doc;
            Document d = searcher.doc(docId);

            result.add(d.get("image"));
            tags.add(d.get("tags"));
            String tagsContained = "";

            String[] imgTags = d.get("tags").split(" ");
            for(int k = 0; k < imgTags.length; k++){
                for(int j = 0; j < tagTerms.length; j++){
                    if(imgTags[k].equals(tagTerms[j])){
                        tagsContained += imgTags[k] + " ";
                    }
                }
            }
            System.out.println("\t" + (i+1) + " &  & " + tagsContained + " \\\\");
            //System.out.println(d.get("image") + " with tags " + d.get("tags"));
        }
        System.out.println("\t\\hline");
        System.out.println("\\end{longtable}");

        if(print){
            Gallery.printPictures(result,tags);
        }
    }

    public static List<String> searchAbstracts(String input, int hitsPerPage) throws ParseException, IOException {
        String query = input;
        WhitespaceAnalyzer analyzer = new WhitespaceAnalyzer(Version.LUCENE_41);
        Query q = new QueryParser(Version.LUCENE_41, "article", analyzer).parse(query);
        Directory index = new NIOFSDirectory(new File(TFIDF_ABSTRACTS_INDEX_STOPWORD));
        IndexReader reader = IndexReader.open(index);
        List<String> resultSet = new ArrayList();

        IndexSearcher searcher = new IndexSearcher(reader);

        //Set similarity model (commenting out both will set default vector model)
        //searcher.setSimilarity(new BM25Similarity());
        //searcher.setSimilarity(new LMJelinekMercerSimilarity((float)0.1));

        TopScoreDocCollector collector = TopScoreDocCollector.create(hitsPerPage, true);
        searcher.search(q, collector);
        ScoreDoc[] hits = collector.topDocs().scoreDocs;

        //System.out.println("Found " + hits.length + " articles.");

        for(int i=0;i<hits.length;++i) {
            int docId = hits[i].doc;
            Document d = searcher.doc(docId);

            resultSet.add(d.get("desc"));
            //System.out.println((i + 1) + ". " + d.get("article") + " :: Words from abstract: " + d.get("desc"));
        }
        return resultSet;
    }
}
