package main.java;

import org.apache.commons.lang3.StringEscapeUtils;
import org.tartarus.snowball.ext.PorterStemmer;
import rita.wordnet.RiWordnet;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Utils {
    private static final String STOPWORDS_PATH = "/Users/Geir/Development/master/src/main/resources/stopwords.txt";
    private List<String> stopwords = new ArrayList<String>();
    public static final double EPSILON = 0.000001;

    public Utils() throws IOException{
        stopwords.addAll(createStopWordList());
    }

    public static void printUniqueTerms(String terms){
        Set<String> termsList = new HashSet<String>();
        String[] list = terms.split(" ");
        for(String word: list){
            termsList.add(word);
        }
        for(String word: termsList){
            System.out.println(word + " ");
        }
        //System.out.println("Total: " + termsList.size() + " words.");
    }

    public static void printList(List<String> rankedResultList) {
        for(String item: rankedResultList){
            if(item.equals(rankedResultList.get(rankedResultList.size()-1))){
                System.out.println(item);
            }
            else{
                System.out.print(item + ", ");
            }
        }
        System.out.println("");
    }

    public static List<String> stringToSortedList(String input){
        List<String> resultList = new ArrayList<String>();
        String[] words = input.split("\\s+");

        for(int i = 0; i < words.length; i++){
            if(ratioOfDigits(words[i]) < 0.2){
                if(!words[i].contains("http://dbpedia.org/ontology/abstract")){
                    resultList.add(words[i]);
                }
            }
        }

        Collections.sort(resultList);

        return resultList;
    }

    public static List<String> sortListByFrequency(List<String> baseList, int sizeOfResultList){
        List<String> result = new ArrayList<String>();
        Map<String, Integer> map = new HashMap<String, Integer>();
        IntValueComparator valueComparator = new IntValueComparator(map);
        TreeMap<String, Integer> sortedMap = new TreeMap<String, Integer>(valueComparator);

        for(String word: baseList){
            Integer frequency = map.get(word);
            map.put(word, (frequency == null) ? 1 : frequency + 1);
        }

        sortedMap.putAll(map);

        //System.out.println("\nThe search resulted in the following suggested tags (Tag : Number of occurrences):");

        for(Map.Entry<String, Integer> item: sortedMap.entrySet()){
            System.out.println(item.getKey() + ": " + item.getValue());
        }

        for(String key: sortedMap.keySet()){
            result.add(key);
            if(result.size() >= sizeOfResultList){
                break;
            }
        }

        return result;
    }

    public static String listToString(List<String> base){
        String result = "";
        for(String item: base){
            result += item + " ";
        }
        if(result.length() > 1){
            return result.substring(0, result.length()-1);
        }
        else{
            return result;
        }
    }

    public static String percentageCompleteCalc(double numberOfLinesSoFar, String percentComplete) {
        double newPercentComplete = (numberOfLinesSoFar / (double)4000000) * 100;
        DecimalFormat format = new DecimalFormat("#.00");
        if(!percentComplete.equals(format.format(newPercentComplete))){
            percentComplete = format.format(newPercentComplete);
            System.out.println(format.format(newPercentComplete) + " percent complete..");
        }
        return percentComplete;
    }

    public static String percentageCompleteCalc(double fileNumber, String percentComplete, File subDirectory) {
        double newPercentComplete = (fileNumber / (double)1000000) * 100;
        DecimalFormat format = new DecimalFormat("#.00");
        if(!percentComplete.equals(format.format(newPercentComplete))){
            percentComplete = format.format(newPercentComplete);
            System.out.println(subDirectory.getName() + ": " + format.format(newPercentComplete) + " percent complete.. (" + fileNumber + " files processed)");
        }
        return percentComplete;
    }

    public static List<String> createStopWordList() throws IOException {
        File stopwordFile = new File(STOPWORDS_PATH);
        List<String> stopWordList = new ArrayList<String>();
        String strLine;

        FileInputStream fis;
        DataInputStream dis;
        BufferedReader br = null;

        try{
            fis = new FileInputStream(stopwordFile.getAbsolutePath());
            dis = new DataInputStream(fis);
            br = new BufferedReader(new InputStreamReader(dis));
        } catch(IOException e){
            System.out.println("** IO ERROR **: " + e.getMessage());
        }

        while((strLine = br.readLine()) != null){
            stopWordList.add(strLine);
        }
        return stopWordList;
    }

    public static boolean lineIsNotCommentOrLineChange(String strLine) {
        return (strLine.charAt(0) != '#') && ((strLine.charAt(0) != '\n'));
    }

    public static String stemWord(String word){
        PorterStemmer stemmer = new PorterStemmer();
        stemmer.setCurrent(word);
        stemmer.stem();
        return stemmer.getCurrent();
    }

    public boolean validateWord(String word){
        if(stopwords.contains(word)){
            return false;
        }
        else if(word.equals("")){
            return false;
        }
        else if(word.length() <= 2){
            return false;
        }
        else if(ratioOfDigits(word) >= 0.1){
            return false;
        }
        return true;
    }

    public static String removeURLs(String input){
        int endOfURL;

        while(input.contains("<http://")){
            endOfURL = input.indexOf(">");
            input = input.substring(endOfURL+1, input.length());

            //System.out.println("URL-removal: Input is now :" + input);
        }
        return input;
    }

    public static String cleanWord(String word) {
        word = word.replace("\\\"", "");
        word = word.replace("\"", "");
        word = word.replace("(", "");
        word = word.replace(")", "");
        word = word.replace("@en", "");
        word = word.replace("[", "");
        word = word.replace("]", "");
        word = word.replace("'s", "");

        if(hasMalplacedPunctuation(word)){
            word = word.substring(0,word.length()-1);
        }
        if(word.length() != 0){
            if((ratioOfDigits(word)/word.length()) > 0.5){
                word = "";
            }
        }
        if(word.length() < 2){
            word = "";
        }
        return word;
    }

    public static <T extends Comparable<? super T>> List<T> asSortedList(Collection<T> c) {
        List<T> list = new ArrayList<T>(c);
        java.util.Collections.sort(list);
        return list;
    }

    private static double ratioOfDigits(String word){
        double digits = 0.0;

        Pattern pattern = Pattern.compile("\\d");
        Matcher matcher = pattern.matcher(word);
        while(matcher.find()){
            digits++;
        }
        return (digits/word.length());
    }

    public static String convertUnicode(String word){
        try{
            word = StringEscapeUtils.unescapeJava(word);
        } catch (IllegalArgumentException e){
            System.out.println("ERROR in converting unicode for: " + word);
            return "";
        }
        return word;
    }

    public static boolean hasMalplacedPunctuation(String word){
        return word.endsWith(".") ||
                word.endsWith(",") ||
                word.endsWith(":") ||
                word.endsWith("!") ||
                word.endsWith(";") ||
                word.endsWith("?") ||
                word.endsWith("'") ||
                word.startsWith("'");
    }

    public static List<String> getHyponymsOrSimilar(String word){
        RiWordnet wordnet = new RiWordnet(null);
        List<String> output = new ArrayList<String>();

        if(wordnet.exists(word)){
            String pos = wordnet.getBestPos(word);
            String[] returnset = null;
            output.add(word);

            if(pos != null){
                if(pos.equals("a")){
                    returnset = wordnet.getAllSimilar(word, pos);
                }
                else if(pos.equals("n")){
                    returnset = wordnet.getAllHyponyms(word, pos);
                }
                if(returnset != null){
                    Collections.addAll(output, returnset);
                }
            }
        }
        else{
            System.out.println("ERROR: " + word + " is not a word that exists");
            output.add("ERROR");
        }
        System.out.println("Hyponyms for " + word + ": ");
        printList(output);

        return output;
    }

    public static List<String> splitWords(List<String> abstractResult) {
        List<String> result = new ArrayList<String>();
        for(String line: abstractResult){
            result.addAll(stringToSortedList(line));
        }
        return result;
    }


    public static void printSet(Set<String> hyponymHierarchy) {
        System.out.println("Hyponym hierarchy: ");
        for(String term: hyponymHierarchy){
            System.out.println(term);
        }
    }

    public static List<String> getUniqueWordsList(List<String> abstractResult) {
        List<String> words = new ArrayList<String>();
        for(String line: abstractResult){
            String[] terms = line.split(" ");
            for(String term: terms){
                if(!words.contains(term)){
                    words.add(term);
                }
            }
        }
        return words;
    }

    public static double logWithoutNaN(double value) {
        if (value == 0) {
            return Math.log(EPSILON);
        } else if (value < 0) {
            return 0;
        }
        return Math.log(value);
    }

    public static boolean containsDigits(String input){
        for (int i = 0, len = input.length(); i < len; i++) {
            if (Character.isDigit(input.charAt(i))) {
                return true;
            }
        }
        return false;
    }
}
